@@ -1,8 +1,13 @@
+#!/usr/bin/env python3
 import gi
-
 gi.require_version("Gtk", "3.0")
-gi.require_version('Budgie', '1.0')
-from gi.repository import Gdk, Gtk, GObject, GdkPixbuf, Budgie, Gio
+gi.require_version('Libxfce4windowing', '0.0')
+from gi.repository import Libxfce4windowing
+if Libxfce4windowing.windowing_get() == Libxfce4windowing.Windowing.WAYLAND:
+    gi.require_version('Budgie', '3.0')
+else:
+    gi.require_version('Budgie', '2.0')
+from gi.repository import Gdk, Gtk, GObject, GdkPixbuf, Budgie, Gio, GLib
 import os
 import time
 from threading import Thread
@@ -15,7 +20,7 @@ import cwtools as cw
 """
 ClockWorks
 Author: Jacob Vlijm
-Copyright © 2017-2021 Ubuntu Budgie Developers
+Copyright © 2017 Ubuntu Budgie Developers
 Website=https://ubuntubudgie.org
 This program is free software: you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -44,7 +49,6 @@ for dr in [hrs_path, mins_path]:
 
 hours = []
 minutes = []
-tz_data = cw.tz_data
 settings = cw.settings
 
 # create images
@@ -187,8 +191,13 @@ class BudgieClockWorksApplet(Budgie.Applet):
         icon = Gtk.Image.new_from_icon_name(
             "budgie-clockworks-symbolic", Gtk.IconSize.MENU
         )
+        self.tz_data = self.get_timerange()
         self.provider = Gtk.CssProvider.new()
         self.provider.load_from_data(clw_css_data.encode())
+        # setup watching applet presence
+        self.currpanelsubject_settings = None
+        GLib.timeout_add_seconds(1, self.watchout)
+        self.clockworks_onpanel = True
         # maingrid
         self.maingrid = Gtk.Grid()
         self.maingrid.set_row_spacing(2)
@@ -239,6 +248,35 @@ class BudgieClockWorksApplet(Budgie.Applet):
         self.update.setDaemon(True)
         self.update.start()
 
+    def get_timerange(self):
+        timelabels = []
+        offset = -86400
+        while offset < 86401:
+            timelabels.append(self.convert_offset_tolabel(offset))
+            offset = offset + 900
+        return timelabels
+
+    def watchout(self):
+        path = "com.solus-project.budgie-panel"
+        panelpath_prestring = "/com/solus-project/budgie-panel/panels/"
+        panel_settings = Gio.Settings.new(path)
+        allpanels_list = panel_settings.get_strv("panels")
+        for p in allpanels_list:
+            panelpath = panelpath_prestring + "{" + p + "}/"
+            self.currpanelsubject_settings = Gio.Settings.new_with_path(
+                path + ".panel", panelpath
+            )
+            applets = self.currpanelsubject_settings.get_strv("applets")
+            if self.uuid in applets:
+                self.currpanelsubject_settings.connect(
+                    "changed", self.check_ifonpanel
+                )
+        return False
+
+    def check_ifonpanel(self, *args):
+        applets = self.currpanelsubject_settings.get_strv("applets")
+        self.clockworks_onpanel = self.uuid in applets
+
     def do_get_settings_ui(self):
         """Return the applet settings with given uuid"""
         return ClockWorksSettings(self.get_applet_settings(self.uuid))
@@ -289,7 +327,9 @@ class BudgieClockWorksApplet(Budgie.Applet):
     def convert_offset_tolabel(self, offset):
         if offset != 0:
             prestring = "-" if offset < 0 else "+"
-            set_time_hrs = self.double_digits(str(int(abs(offset) / 3600)))
+            # calc with abs!
+            offset = abs(offset)
+            set_time_hrs = self.double_digits(str(int(offset / 3600)))
             set_time_mins = self.double_digits(str(int((offset % 3600) / 60)))
             return prestring + set_time_hrs + ":" + set_time_mins
         else:
@@ -299,7 +339,8 @@ class BudgieClockWorksApplet(Budgie.Applet):
         newval = label.split(":")
         hrs_offset = int(newval[0]) * 3600
         mins_offset = int(newval[1]) * 60
-        mins_offset = mins_offset if hrs_offset > 0 else -mins_offset
+        if label.startswith("-"):
+            mins_offset = -mins_offset
         return (hrs_offset + mins_offset)
 
     def save_tofile(self):
@@ -447,7 +488,7 @@ class BudgieClockWorksApplet(Budgie.Applet):
 
     def update_gmt(self):
         # The loop, sleep is set automaically (appr 1 minute after first cycle)
-        while True:
+        while self.clockworks_onpanel:
             timedata = self.waitfornext()
             curr_time = timedata[0]
             self.refresh_clocks(curr_time)
@@ -465,14 +506,14 @@ class BudgieClockWorksApplet(Budgie.Applet):
         # sets the time shift on combo change
         curr_offset = self.clocklist[clock]["offset"]
         nxt = -1 if down else +1
-        newindex = tz_data.index(
+        newindex = self.tz_data.index(
             self.convert_offset_tolabel(curr_offset)
         ) + nxt
         newindex = 0 if newindex < 0 else newindex
         try:
-            newlabel = tz_data[newindex]
+            newlabel = self.tz_data[newindex]
         except IndexError:
-            newlabel = tz_data[-1]
+            newlabel = self.tz_data[-1]
         label.set_text(newlabel)
         self.clocklist[clock]["offset"] = self.convert_label_tooffset(newlabel)
         curr_time = time.time()
